@@ -1,21 +1,23 @@
+from typing import Any, List
+
 import django
-from django.core.checks import Tags, Warning, register
+from django.core import checks
 
 from django_mysql.utils import mysql_connections
 
 
-def register_checks():
-    register(Tags.database)(check_variables)
+def register_checks() -> None:
+    checks.register(checks.Tags.database)(check_variables)
 
 
-def check_variables(app_configs, **kwargs):
+def check_variables(**kwargs: Any) -> List[checks.CheckMessage]:
     if django.VERSION >= (3, 1):
         # when moving to Django 3.1+ only support, make this a real argument
         databases = kwargs["databases"]
     else:
         databases = {alias for alias, connection in mysql_connections()}
 
-    errors = []
+    errors: List[checks.CheckMessage] = []
 
     if databases is None:
         return errors
@@ -27,16 +29,11 @@ def check_variables(app_configs, **kwargs):
 
         with connection.temporary_connection() as cursor:
             cursor.execute(
-                """SELECT @@sql_mode,
-                          @@innodb_strict_mode,
+                """SELECT @@innodb_strict_mode,
                           @@character_set_connection"""
             )
             variables = cursor.fetchone()
-            sql_mode, innodb_strict_mode, character_set_connection = variables
-
-        modes = set(sql_mode.split(","))
-        if not (modes & {"STRICT_TRANS_TABLES", "STRICT_ALL_TABLES"}):
-            errors.append(strict_mode_warning(alias))
+            innodb_strict_mode, character_set_connection = variables
 
         if not innodb_strict_mode:
             errors.append(innodb_strict_mode_warning(alias))
@@ -47,22 +44,9 @@ def check_variables(app_configs, **kwargs):
     return errors
 
 
-def strict_mode_warning(alias):
-    return Warning(
-        "MySQL Strict Mode is not set for database connection '{}'".format(alias),
-        hint=(
-            "MySQL's Strict Mode fixes many data integrity problems in MySQL, "
-            + "such as data truncation upon insertion, by escalating warnings "
-            + "into errors. It is strongly recommended you activate it. See: "
-            + "https://django-mysql.readthedocs.io/en/latest/checks.html#django-mysql-w001-strict-mode"  # noqa: B950
-        ),
-        id="django_mysql.W001",
-    )
-
-
-def innodb_strict_mode_warning(alias):
-    return Warning(
-        "InnoDB Strict Mode is not set for database connection '{}'".format(alias),
+def innodb_strict_mode_warning(alias: str) -> checks.Warning:
+    return checks.Warning(
+        f"InnoDB Strict Mode is not set for database connection '{alias}'",
         hint=(
             "InnoDB Strict Mode escalates several warnings around "
             + "InnoDB-specific statements into errors. It's recommended you "
@@ -74,9 +58,9 @@ def innodb_strict_mode_warning(alias):
     )
 
 
-def utf8mb4_warning(alias):
-    return Warning(
-        "The character set is not utf8mb4 for database connection '{}'".format(alias),
+def utf8mb4_warning(alias: str) -> checks.Warning:
+    return checks.Warning(
+        f"The character set is not utf8mb4 for database connection '{alias}'",
         hint=(
             "The default 'utf8' character set does not include support for "
             + "all Unicode characters. It's strongly recommended you move to "
