@@ -1,67 +1,43 @@
+import datetime as dt
 import json
-import warnings
+from typing import Any, Dict, List, Optional, Tuple, Type, Union
 
 from django.db import DEFAULT_DB_ALIAS, connections
-from django.db.models import CharField
+from django.db.models import CharField, Expression
 from django.db.models import Field as DjangoField
 from django.db.models import Func, IntegerField, TextField, Value
 
+from django_mysql.compat import HAVE_JSONFIELD, JSONField
+
+ExpressionArgument = Union[
+    Expression,
+    str,  # column reference handled by Django
+]
+
 
 class SingleArgFunc(Func):
 
-    output_field_class = None
+    output_field_class: Optional[Type[DjangoField]] = None
 
-    def __init__(self, expression):
+    def __init__(self, expression: ExpressionArgument) -> None:
         super().__init__(expression)
         if self.output_field_class is not None:
             self.output_field = self.output_field_class()
 
 
-class MultiArgFunc(Func):
-    def __init__(self, *expressions):
-        super().__init__(*expressions)
-
-
-# Comparison Functions
-
-
-class Greatest(MultiArgFunc):
-    function = "GREATEST"
-
-    def __init__(self, *args, **kwargs):
-        warnings.warn(
-            (
-                "This function is deprecated. Use "
-                + "django.db.models.functions.Greatest instead."
-            ),
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        super().__init__(*args, **kwargs)
-
-
-class Least(MultiArgFunc):
-    function = "LEAST"
-
-    def __init__(self, *args, **kwargs):
-        warnings.warn(
-            (
-                "This function is deprecated. Use "
-                + "django.db.models.functions.Least instead."
-            ),
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        super().__init__(*args, **kwargs)
-
-
 # Control Flow Functions
 
 
 class If(Func):
     function = "IF"
 
-    def __init__(self, condition, true, false=None, output_field=None):
+    def __init__(
+        self,
+        condition: ExpressionArgument,
+        true: ExpressionArgument,
+        false: Optional[ExpressionArgument] = None,
+        output_field: Optional[DjangoField] = None,
+    ) -> None:
         if output_field is None:
             # Workaround for some ORM weirdness
             output_field = DjangoField()
@@ -72,68 +48,11 @@ class If(Func):
 # Numeric Functions
 
 
-class Abs(SingleArgFunc):
-    function = "ABS"
-    output_field_class = IntegerField
-
-    def __init__(self, *args, **kwargs):
-        warnings.warn(
-            "This function is deprecated. Use django.db.models.functions.Abs instead.",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        super().__init__(*args, **kwargs)
-
-
-class Ceiling(SingleArgFunc):
-    function = "CEILING"
-    output_field_class = IntegerField
-
-    def __init__(self, *args, **kwargs):
-        warnings.warn(
-            "This function is deprecated. Use django.db.models.functions.Ceil instead.",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        super().__init__(*args, **kwargs)
-
-
 class CRC32(SingleArgFunc):
     function = "CRC32"
     output_field_class = IntegerField
 
 
-class Floor(SingleArgFunc):
-    function = "FLOOR"
-    output_field_class = IntegerField
-
-    def __init__(self, *args, **kwargs):
-        warnings.warn(
-            (
-                "This function is deprecated. Use "
-                + "django.db.models.functions.Floor instead."
-            ),
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        super().__init__(*args, **kwargs)
-
-
-class Round(Func):
-    function = "ROUND"
-
-    def __init__(self, expression, places=0):
-        warnings.warn(
-            (
-                "This function is deprecated. Use "
-                + "django.db.models.functions.Round instead."
-            ),
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        super().__init__(expression, places, output_field=IntegerField())
-
-
 class Sign(SingleArgFunc):
     function = "SIGN"
     output_field_class = IntegerField
@@ -149,15 +68,9 @@ class ConcatWS(Func):
 
     function = "CONCAT_WS"
 
-    def __init__(self, *expressions, **kwargs):
-        separator = kwargs.pop("separator", ",")
-        if len(kwargs) > 0:
-            raise ValueError(
-                "Invalid keyword arguments for ConcatWS: {}".format(
-                    ",".join(kwargs.keys())
-                )
-            )
-
+    def __init__(
+        self, *expressions: ExpressionArgument, separator: Optional[str] = ","
+    ) -> None:
         if len(expressions) < 2:
             raise ValueError("ConcatWS must take at least two expressions")
 
@@ -172,7 +85,11 @@ class ConcatWS(Func):
 class ELT(Func):
     function = "ELT"
 
-    def __init__(self, num, expressions):
+    def __init__(
+        self,
+        num: ExpressionArgument,
+        expressions: Union[List[ExpressionArgument], Tuple[ExpressionArgument]],
+    ) -> None:
         value_exprs = []
         for v in expressions:
             if not hasattr(v, "resolve_expression"):
@@ -185,7 +102,12 @@ class ELT(Func):
 class Field(Func):
     function = "FIELD"
 
-    def __init__(self, field, values, **kwargs):
+    def __init__(
+        self,
+        field: ExpressionArgument,
+        values: Union[List[ExpressionArgument], Tuple[ExpressionArgument]],
+        **kwargs: Any,
+    ) -> None:
         values_exprs = []
         for v in values:
             if not hasattr(v, "resolve_expression"):
@@ -201,7 +123,12 @@ class Field(Func):
 class UpdateXML(Func):
     function = "UPDATEXML"
 
-    def __init__(self, xml_target, xpath_expr, new_xml):
+    def __init__(
+        self,
+        xml_target: ExpressionArgument,
+        xpath_expr: ExpressionArgument,
+        new_xml: ExpressionArgument,
+    ) -> None:
         if not hasattr(xpath_expr, "resolve_expression"):
             xpath_expr = Value(xpath_expr)
         if not hasattr(new_xml, "resolve_expression"):
@@ -215,7 +142,9 @@ class UpdateXML(Func):
 class XMLExtractValue(Func):
     function = "EXTRACTVALUE"
 
-    def __init__(self, xml_frag, xpath_expr):
+    def __init__(
+        self, xml_frag: ExpressionArgument, xpath_expr: ExpressionArgument
+    ) -> None:
         if not hasattr(xpath_expr, "resolve_expression"):
             xpath_expr = Value(xpath_expr)
 
@@ -239,7 +168,7 @@ class SHA2(Func):
     function = "SHA2"
     hash_lens = (224, 256, 384, 512)
 
-    def __init__(self, expression, hash_len=512):
+    def __init__(self, expression: ExpressionArgument, hash_len: int = 512) -> None:
         if hash_len not in self.hash_lens:
             raise ValueError(
                 "hash_len must be one of {}".format(
@@ -255,7 +184,7 @@ class SHA2(Func):
 class LastInsertId(Func):
     function = "LAST_INSERT_ID"
 
-    def __init__(self, expression=None):
+    def __init__(self, expression: Optional[ExpressionArgument] = None) -> None:
         if expression is not None:
             super().__init__(expression)
         else:
@@ -264,7 +193,7 @@ class LastInsertId(Func):
         self.output_field = IntegerField()
 
     @classmethod
-    def get(cls, using=DEFAULT_DB_ALIAS):
+    def get(cls, using: str = DEFAULT_DB_ALIAS) -> int:
         # N.B. did try getting it from connection.connection.insert_id() (The
         # MySQLdb query-free method) but it did not work with non-default
         # database connections in Django, and the reason was not clear
@@ -275,107 +204,122 @@ class LastInsertId(Func):
 
 # JSON Functions
 
+if HAVE_JSONFIELD:
 
-class JSONExtract(Func):
-    function = "JSON_EXTRACT"
+    class JSONExtract(Func):
+        function = "JSON_EXTRACT"
 
-    def __init__(self, expression, *paths, output_field=None):
-        from django_mysql.models.fields import JSONField
+        def __init__(
+            self,
+            expression: ExpressionArgument,
+            *paths: ExpressionArgument,
+            output_field: Optional[Type[DjangoField]] = None,
+        ) -> None:
+            exprs = [expression]
+            for path in paths:
+                if not hasattr(path, "resolve_expression"):
+                    path = Value(path)
+                exprs.append(path)
 
-        exprs = [expression]
-        for path in paths:
-            if not hasattr(path, "resolve_expression"):
-                path = Value(path)
-            exprs.append(path)
+            if output_field is not None:
+                if len(paths) > 1:
+                    raise TypeError(
+                        "output_field won't work with more than one path, as a "
+                        "JSON Array will be returned"
+                    )
+            else:
+                output_field = JSONField()
 
-        if output_field is not None:
-            if len(paths) > 1:
-                raise TypeError(
-                    "output_field won't work with more than one path, as a "
-                    "JSON Array will be returned"
-                )
-        else:
-            output_field = JSONField()
+            super().__init__(*exprs, output_field=output_field)
 
-        super().__init__(*exprs, output_field=output_field)
+    class JSONKeys(Func):
+        function = "JSON_KEYS"
 
+        def __init__(
+            self,
+            expression: ExpressionArgument,
+            path: Optional[ExpressionArgument] = None,
+        ) -> None:
+            exprs = [expression]
+            if path is not None:
+                if not hasattr(path, "resolve_expression"):
+                    path = Value(path)
+                exprs.append(path)
 
-class JSONKeys(Func):
-    function = "JSON_KEYS"
+            super().__init__(*exprs, output_field=JSONField())
 
-    def __init__(self, expression, path=None):
-        from django_mysql.models.fields import JSONField
+    class JSONLength(Func):
+        function = "JSON_LENGTH"
 
-        exprs = [expression]
-        if path is not None:
-            if not hasattr(path, "resolve_expression"):
-                path = Value(path)
-            exprs.append(path)
+        def __init__(
+            self,
+            expression: ExpressionArgument,
+            path: Optional[ExpressionArgument] = None,
+            *,
+            output_field: Optional[DjangoField] = None,
+            **extra: Any,
+        ) -> None:
+            if output_field is None:
+                output_field = IntegerField()
 
-        super().__init__(*exprs, output_field=JSONField())
+            exprs = [expression]
+            if path is not None:
+                if not hasattr(path, "resolve_expression"):
+                    path = Value(path)
+                exprs.append(path)
 
+            super().__init__(*exprs, output_field=output_field)
 
-class JSONLength(Func):
-    function = "JSON_LENGTH"
+    class JSONValue(Func):
+        function = "CAST"
+        template = "%(function)s(%(expressions)s AS JSON)"
 
-    def __init__(self, expression, path=None, **extra):
-        output_field = extra.pop("output_field", IntegerField())
+        def __init__(
+            self, expression: Union[None, int, float, str, List[Any], Dict[str, Any]]
+        ) -> None:
+            json_string = json.dumps(expression, allow_nan=False)
+            super().__init__(Value(json_string))
 
-        exprs = [expression]
-        if path is not None:
-            if not hasattr(path, "resolve_expression"):
-                path = Value(path)
-            exprs.append(path)
+    class BaseJSONModifyFunc(Func):
+        def __init__(
+            self,
+            expression: ExpressionArgument,
+            data: Dict[
+                str,
+                Union[
+                    ExpressionArgument, None, int, float, str, List[Any], Dict[str, Any]
+                ],
+            ],
+        ) -> None:
+            if not data:
+                raise ValueError('"data" cannot be empty')
 
-        super().__init__(*exprs, output_field=output_field)
+            exprs = [expression]
 
+            for path, value in data.items():
+                if not hasattr(path, "resolve_expression"):
+                    path = Value(path)
 
-class JSONValue(Func):
-    function = "CAST"
-    template = "%(function)s(%(expressions)s AS JSON)"
+                exprs.append(path)
 
-    def __init__(self, expression):
-        json_string = json.dumps(expression, allow_nan=False)
-        super().__init__(Value(json_string))
+                if not hasattr(value, "resolve_expression"):
+                    value = JSONValue(value)
 
+                exprs.append(value)
 
-class BaseJSONModifyFunc(Func):
-    def __init__(self, expression, data):
-        from django_mysql.models.fields import JSONField
+            super().__init__(*exprs, output_field=JSONField())
 
-        if not data:
-            raise ValueError('"data" cannot be empty')
+    class JSONInsert(BaseJSONModifyFunc):
+        function = "JSON_INSERT"
 
-        exprs = [expression]
+    class JSONReplace(BaseJSONModifyFunc):
+        function = "JSON_REPLACE"
 
-        for path, value in data.items():
-            if not hasattr(path, "resolve_expression"):
-                path = Value(path)
+    class JSONSet(BaseJSONModifyFunc):
+        function = "JSON_SET"
 
-            exprs.append(path)
-
-            if not hasattr(value, "resolve_expression"):
-                value = JSONValue(value)
-
-            exprs.append(value)
-
-        super().__init__(*exprs, output_field=JSONField())
-
-
-class JSONInsert(BaseJSONModifyFunc):
-    function = "JSON_INSERT"
-
-
-class JSONReplace(BaseJSONModifyFunc):
-    function = "JSON_REPLACE"
-
-
-class JSONSet(BaseJSONModifyFunc):
-    function = "JSON_SET"
-
-
-class JSONArrayAppend(BaseJSONModifyFunc):
-    function = "JSON_ARRAY_APPEND"
+    class JSONArrayAppend(BaseJSONModifyFunc):
+        function = "JSON_ARRAY_APPEND"
 
 
 # MariaDB Regexp Functions
@@ -384,7 +328,9 @@ class JSONArrayAppend(BaseJSONModifyFunc):
 class RegexpInstr(Func):
     function = "REGEXP_INSTR"
 
-    def __init__(self, expression, regex):
+    def __init__(
+        self, expression: ExpressionArgument, regex: ExpressionArgument
+    ) -> None:
         if not hasattr(regex, "resolve_expression"):
             regex = Value(regex)
 
@@ -394,7 +340,12 @@ class RegexpInstr(Func):
 class RegexpReplace(Func):
     function = "REGEXP_REPLACE"
 
-    def __init__(self, expression, regex, replace):
+    def __init__(
+        self,
+        expression: ExpressionArgument,
+        regex: ExpressionArgument,
+        replace: ExpressionArgument,
+    ) -> None:
         if not hasattr(regex, "resolve_expression"):
             regex = Value(regex)
 
@@ -407,7 +358,9 @@ class RegexpReplace(Func):
 class RegexpSubstr(Func):
     function = "REGEXP_SUBSTR"
 
-    def __init__(self, expression, regex):
+    def __init__(
+        self, expression: ExpressionArgument, regex: ExpressionArgument
+    ) -> None:
         if not hasattr(regex, "resolve_expression"):
             regex = Value(regex)
 
@@ -425,17 +378,17 @@ class AsType(Func):
     function = ""
     template = "%(expressions)s AS %(data_type)s"
 
-    def __init__(self, expression, data_type):
+    def __init__(self, expression: ExpressionArgument, data_type: str) -> None:
         if not hasattr(expression, "resolve_expression"):
             expression = Value(expression)
 
         if data_type not in self.TYPE_MAP:
-            raise ValueError("Invalid data_type '{}'".format(data_type))
+            raise ValueError(f"Invalid data_type '{data_type}'")
 
         super().__init__(expression, data_type=data_type)
 
     @property
-    def TYPE_MAP(self):
+    def TYPE_MAP(self) -> Dict[str, Union[Type[DjangoField], DjangoField]]:
         from django_mysql.models.fields.dynamic import KeyTransform
 
         return KeyTransform.TYPE_MAP
@@ -444,7 +397,13 @@ class AsType(Func):
 class ColumnAdd(Func):
     function = "COLUMN_ADD"
 
-    def __init__(self, expression, to_add):
+    def __init__(
+        self,
+        expression: ExpressionArgument,
+        to_add: Dict[
+            str, Union[ExpressionArgument, float, int, dt.date, dt.time, dt.datetime]
+        ],
+    ) -> None:
         from django_mysql.models.fields import DynamicField
 
         expressions = [expression]
@@ -465,7 +424,9 @@ class ColumnAdd(Func):
 class ColumnDelete(Func):
     function = "COLUMN_DELETE"
 
-    def __init__(self, expression, *to_delete):
+    def __init__(
+        self, expression: ExpressionArgument, *to_delete: ExpressionArgument
+    ) -> None:
         from django_mysql.models.fields import DynamicField
 
         expressions = [expression]
@@ -481,14 +442,19 @@ class ColumnGet(Func):
     function = "COLUMN_GET"
     template = "COLUMN_GET(%(expressions)s AS %(data_type)s)"
 
-    def __init__(self, expression, column_name, data_type):
+    def __init__(
+        self,
+        expression: ExpressionArgument,
+        column_name: ExpressionArgument,
+        data_type: ExpressionArgument,
+    ):
         if not hasattr(column_name, "resolve_expression"):
             column_name = Value(column_name)
 
         try:
             output_field = self.TYPE_MAP[data_type]
         except KeyError:
-            raise ValueError("Invalid data_type '{}'".format(data_type))
+            raise ValueError(f"Invalid data_type '{data_type}'")
 
         if data_type == "BINARY":
             output_field = output_field()
@@ -498,7 +464,7 @@ class ColumnGet(Func):
         )
 
     @property
-    def TYPE_MAP(self):
+    def TYPE_MAP(self) -> Dict[str, Union[DjangoField, Type[DjangoField]]]:
         from django_mysql.models.fields.dynamic import KeyTransform
 
         return KeyTransform.TYPE_MAP
